/*
 * Belegg2.java   E.L. 2002-09-04
 *
 * Klasser for  illustrere arv i to niver.
 */

/*
 * Vi lager denne klassen abstrakt. Den har imidlertid ingen abstrakte metoder.
 */
abstract class Belegg2 extends Materiale {
  private static final double grense = 0.02; // grense for  beregne en bredde ekstra
  private double breddePBelegg;    // meter

  /* Konstruktren trengs i subklasser */
  protected Belegg2(String startNavn, double startPris, double startBredde) {
    super(startNavn, startPris);
    breddePBelegg = startBredde;
  }

  public final double finnBredde() {
    return breddePBelegg;
  }

  public double finnMaterialbehov(Flate enFlate) {
    double lengdeFlate = enFlate.finnLengde();
    double breddeFlate = enFlate.finnBredde();
    int antBredder = (int)(lengdeFlate / breddePBelegg);
    double rest = lengdeFlate % breddePBelegg;
    if (rest >= grense) antBredder++;
    return antBredder * breddeFlate;
  }

  public String toString() {
    return "Belegg2: " + super.toString() + " pr. m, bredde: " +
            finnFormat().format(breddePBelegg) + " m.";
  }
}

/*
 * Belegg av frstesortering. Ingen endringer i forhold til klassen Belegg2.
 */
final class ForsteSortBelegg extends Belegg2 {

  public ForsteSortBelegg(String startNavn, double startPris, int startBredde) {
    super(startNavn, startPris, startBredde);
  }
}

/*
 * Belegg av annensortering. Reviderte beregningsalgoritmer for behov og pris.
 */
final class AnnenSortBelegg extends Belegg2 {
  public static final double grenseHyRabatt = 5;
  public static final double hyesteRabatt = 0.7;
  public static final double lavesteRabatt = 0.5;
  public static final double materialTillegg = 1.2; // faktor for materialtillegg

  public AnnenSortBelegg(String startNavn, double startPris, int startBredde) {
    super(startNavn, startPris, startBredde);
  }

  public double finnTotalpris(Flate enFlate) {
    double grunnpris = super.finnTotalpris(enFlate);
    double behov = finnMaterialbehov(enFlate);
    if (behov > grenseHyRabatt) return grunnpris * (1.0 - hyesteRabatt);
    else return grunnpris * (1.0 - lavesteRabatt);
  }

  public double finnMaterialbehov(Flate enFlate) {
    double grunnBehov = super.finnMaterialbehov(enFlate);
    return grunnBehov * materialTillegg;
  }
}