/*
 * Geometritest.java   E.L. 2004-02-15
 *
 * Filen inneholder fem klasser:
 * Figur: En abstrakt klasse med den abstrakte operasjonen finnAreal()
 * Sirkel, Trekant og Kvadrat: Subklasser til Figur,
 *         med hver sin implementasjon av finnAreal()
 * Geometritest: Lager en tabell av figurer, som viser at polymorfien virker
 */

abstract class Figur {
  /* Alle figurer kan beregne sitt eget areal */
  public abstract double finnAreal();
}

/*
 * Kvadrat: Arealet er lik side * side
 */
class Kvadrat extends Figur {
  private double side;

  public Kvadrat(int startSide) {
    side = startSide;
  }

  public double finnSide() {
    return side;
  }

  public double finnAreal() {
    return side * side;
  }
}

/*
 * Trekant: Arealet er lik 0,5 * grunnlinje * hyde
 */
class Trekant extends Figur {
  private double grunnlinje;
  private double hyde;

  public Trekant(double startGrunnlinje, double startHyde) {
    grunnlinje = startGrunnlinje;
    hyde = startHyde;
  }

  public double finnGrunnlinje() {
    return grunnlinje;
  }

  public double finnHyde() {
    return hyde;
  }

  public double finnAreal() {
    return 0.5 * grunnlinje * hyde;
  }
}

/*
 * Sirkel: Arealet er lik pi * radius * radius
 */
class Sirkel extends Figur {

  private double radius;

  public Sirkel(double startRadius) {
    radius = startRadius;
  }

  public double finnRadius() {
    return radius;
  }

  public double finnAreal() {
    return Math.PI * radius * radius ;
  }
}

class Geometritest {
  public static void main(String[] args) {
    Sirkel enSirkel = new Sirkel(10);
    Trekant enTrekant = new Trekant(5, 6);
    Kvadrat etKvadrat = new Kvadrat(4);

    /* Arealer hver for seg */
    System.out.println("Arealet til sirkelen: " + enSirkel.finnAreal());
    System.out.println("Arealet til trekanten: " + enTrekant.finnAreal());
    System.out.println("Arealet til kvadratet: " + etKvadrat.finnAreal());

    /* Behandler s alle figurene under ett */
    Figur[] figurtabell = new Figur[3];
    figurtabell[0] = enSirkel;
    figurtabell[1] = enTrekant;
    figurtabell[2] = etKvadrat;

    for (Figur fig : figurtabell) {
      System.out.println("Arealet er " + fig.finnAreal()
            + " for " + fig.getClass().getName()); // henter klassenavnet
    }
  }
}

/* Utskrift:
Arealet til sirkelen: 314.1592653589793
Arealet til trekanten: 15.0
Arealet til kvadratet: 16.0
Arealet er 314.1592653589793 for Sirkel
Arealet er 15.0 for Trekant
Arealet er 16.0 for Kvadrat
*/
