/*
 * TestAnsatt.java E.L. 2004-02-15
 *
 * Et objekt av klassen Ansatt inneholder ansattnr., navn og timelnn.
 * Konstruktren kaster unntaksobjekt dersom ansattnr. utenfor tillatt omrde
 * eller lnnen er mindre enn en gitt minstelnn.
 */

import mittBibliotek.DataLeser;  // se kapittel 6.8
import java.text.NumberFormat;
import java.util.ArrayList;
import static javax.swing.JOptionPane.*;

class Ansatt {
  private static final int nedreGrenseNr = 1000; // nedre grense, ans.nr.
  private static final int vreGrenseNr = 9999;  // vre grense, ans.nr.
  private static final double minLnn = 100.0;   // minste lnn pr. time

  private int ansNr;
  private String navn;
  private double lnn; // pr. time

  public Ansatt(int startNr, String startNavn, double startLnn)
                                          throws IllegalArgumentException {
    NumberFormat pengeformat = NumberFormat.getCurrencyInstance();
    if (startLnn < minLnn) {
      throw new IllegalArgumentException("Lnn: " + pengeformat.format(startLnn) +
                "\nLnnen skal vre minst " + pengeformat.format(minLnn));
    } else if (startNr < nedreGrenseNr || startNr > vreGrenseNr) {
      throw new IllegalArgumentException("Nummer: " + startNr +
                         "\nAnsattnummeret m ligge i intervallet [" +
                             nedreGrenseNr + ", " + vreGrenseNr + "].");
    } else {
      ansNr = startNr;
      navn = startNavn;
      lnn = startLnn;
    }
  }

  public int finnNr() {
    return ansNr;
  }

  public String finnNavn() {
    return navn;
  }

  public double finnLnn() {
    return lnn;
  }

  public String toString() {
    NumberFormat pengeformat = NumberFormat.getCurrencyInstance();
    return "No. : " + ansNr + ", navn: " + navn + ", lnn: " + pengeformat.format(lnn);
  }
}

class TestAnsatt {
  public static void main(String[] args) {
    /*
     * Leser inn opplysninger om ansatte, lager objekter og lagrer disse i en tabell-liste.
     * Ingen kontroll p at ansatte med dette nummer eller navn er registrert fra fr.
     */
    ArrayList<Ansatt> ansatte = new ArrayList<Ansatt>();
    int ansNr = DataLeser.lesHeltall("Ansatt nr., 4 sifre, 0 for avslutt: ");
    while (ansNr != 0) {
      try {
        String navn = DataLeser.lesTekst("Navn: ");
        double lnn = DataLeser.lesDesimaltall("Timelnn: ");
        Ansatt ans = new Ansatt(ansNr, navn, lnn);
        ansatte.add(ans);
      } catch (IllegalArgumentException e) {
        showMessageDialog(null, e.getMessage() + "\nBegynn p nytt med denne ansatte.");
      }
      ansNr = DataLeser.lesHeltall("Ansatt nr., 4 sifre, 0 for avslutt: ");
    }
    System.out.println("Kontrollutskrift:\n");
    for (Ansatt ans : ansatte) System.out.println(ans);
  }
}