/*
 * Oppussingsprosjekt.java   E.L. 2004-02-15
 *
 * Klassen vedlikeholder et register over flater og et register over malingstyper.
 * Klassen arbeider hele tiden med referanser, den lager ikke p noe tidspunkt
 * kopier av noen av objektene.
 * En klient kan legge inn referanser til objekter.
 * Klienten kan ogs hente ut en referanse til et objekt, og via denne
 * gjre endringer i datainnholdet i objektet, dersom det er mutabelt.
 * Her betyr det konkret at klienten kan hente ut et flate-
 * objekt og endre malingstypen for dette objektet.
 *
 * Klassene Maling og Flate er ikke vist i boka.
 * Se [URL-Javabok] under eksempler fra kap. 9.
 */

import java.util.ArrayList;
import java.text.DecimalFormat;

class Oppussingsprosjekt implements java.io.Serializable {
  public static final DecimalFormat utskriftFormat = new DecimalFormat("####0.00");

  private String navn;
  private ArrayList <Flate>alleFlater = new ArrayList<Flate>();
  private ArrayList <Maling>alleMalingstyper = new ArrayList<Maling>();

  public Oppussingsprosjekt(String startNavn) {
    navn = startNavn;
  }

  public String finnNavn() {
    return navn;
  }

  /*
   * Metoden registrerer en ny malingstype. Dersom en malingstype med dette
   * navnet er registrert fra fr, returneres denne malingstypen,
   * ellers returneres den nye malingstypen.
   */
  public Maling registrerNyMaling(Maling nyMalingstype) {
    String navnet = nyMalingstype.finnNavn();
    Maling denne = finnMaling(navnet);
    if (denne == null) {
      alleMalingstyper.add(nyMalingstype);
      return nyMalingstype;
    }
    else return denne;
  }

  /*
   * Metoden sker etter en malingstype med et gitt navn.
   * Dersom det ikke finnes, returneres null.
   */
  public Maling finnMaling(String malingnavn) {
    for (Maling denne : alleMalingstyper) {
      String denneNavn = denne.finnNavn();
      if (denneNavn.equals(malingnavn)) return denne;
    }
    return null;
  }

  /* Metoden er bygd opp p samme mte som registrerNyMaling() */
  public Flate registrerNyFlate(Flate nyFlate) {
    String navnet = nyFlate.finnNavn();
    Flate denne = finnFlate(navnet);
    if (denne == null) {
      alleFlater.add(nyFlate);
      /*
       * Malingstype kan vre knyttet til denne flaten.
       * Hvis denne malingstypen ikke er registrert her fra fr, registrerer vi den n.
       */
      Maling malingen = nyFlate.finnMaling();
      if (malingen != null) registrerNyMaling(malingen);
      return nyFlate;
    }
    else return denne;  // flate med dette navnet er registrert fra fr
  }

  /* Metoden er bygd opp p samme mte som finnMaling() */
  public Flate finnFlate(String Flatenavn) {
    for (Flate denne : alleFlater) {
      String denneNavn = denne.finnNavn();
      if (denneNavn.equals(Flatenavn)) return denne;
    }
    return null;
  }

  /* Finner prosjektets totalpris */
  public double finnTotalpris() {
    double totalpris = 0.0;
    for (Flate flaten : alleFlater) {
      Maling maling = flaten.finnMaling();
      if (maling != null) totalpris += maling.finnTotalpris(flaten);
    }
    return totalpris;
  }

  /*
   * Metoder som kan brukes ved sekvensiell sk gjennom
   *  alle malinger eller alle flater
   */

  public int finnAntMalinger() {
    return alleMalingstyper.size();
  }

  public Maling finnMaling(int indeks){
    if (indeks >= 0 && indeks < alleMalingstyper.size()) {
      return alleMalingstyper.get(indeks);
    }
    else return null;
  }

  public int finnAntFlater() {
    return alleFlater.size();
  }

  public Flate finnFlate(int indeks){
    if (indeks >= 0 && indeks < alleFlater.size()) {
      return alleFlater.get(indeks);
    }
    else return null;
  }

  public String lagPrisOgBehovDetaljer() {
    String resultat = "";
    double totalpris = 0.0;
    for (Flate flaten : alleFlater) {
      resultat += flaten.toString();
      Maling maling = flaten.finnMaling();
      if (maling != null) {
        double behov = maling.finnAntLiter(flaten);
        double pris = maling.finnTotalpris(flaten);
        resultat += ", behov: " + utskriftFormat.format(behov) +
                    " liter, pris " + utskriftFormat.format(pris) + " kr.\n";
        totalpris += pris;
      } else resultat += " Malingstype ikke bestemt\n";
    }
    resultat += "Totalpris " + utskriftFormat.format(totalpris) + " kr.";
    return resultat;
  }

  public String toString() {
    String resultat = "Alle flater:\n";
    for (Flate enFlate : alleFlater) resultat += enFlate + "\n";

    resultat += "\nAlle malingstyper:\n";
    for (Maling enMaling : alleMalingstyper) resultat += enMaling + "\n";
    return resultat;
  }
}
