/*
 * TestTegneVindu.java  E.L. 2004-02-15
 *
 * Tegner en ellipse der brukeren bestemmer strrelse, plassering og farge.
 * Ingen kontroll p at ellipsen havner innenfor vinduet.
 *
 * Filen inneholder flere klasser:
 * VinduMedtegning: Inneholder tre paneler, ett for inndata, ett for ellipsen, og ett med
 * knapper for fargevalg. Panelene er beskrevet i indre klasser. Vi har ogs en
 * indre klasse som beskriver en knappelytter.
 * Tegning: Tegner en ellipse etter gitte spesifikasjoner.
 * TestTegneVindu: Inneholder main()-metode som viser fram vinduet.
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class VinduMedTegning extends JFrame {

  /* Ellipseattributter som brukes frste gang ellipsen tegnes ut */
  private static final String x = "70";
  private static final String y = "50";
  private static final String bredde = "50";
  private static final String hyde = "60";

  /*
   * Flgende variabler brukes i konstruktren, i lytter-klassen,
   * og i BrukerInputPanel-klassen:
   */
  private Tegning tegningen;
  private JTextField xFelt = new JTextField(x);
  private JTextField yFelt = new JTextField(y);
  private JTextField breddeFelt = new JTextField(bredde);
  private JTextField hydeFelt = new JTextField(hyde);

  public VinduMedTegning(String tittel) {  // Standard layouthndterer BorderLayout
    setTitle(tittel);
    setSize(400, 300); // p grunn av tegningen kan vi ikke bruke pack()
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

    BrukerInputPanel nord = new BrukerInputPanel();
    add(nord, BorderLayout.NORTH);

    tegningen = new Tegning(x, y, bredde, hyde);
    add(tegningen, BorderLayout.CENTER);

    Knappepanel sr = new Knappepanel();
    add(sr, BorderLayout.SOUTH);
  }

  /* Lytter etter knappetrykk */
  private class Knappelytter implements ActionListener {
    public void actionPerformed(ActionEvent hendelse) {

      /* Henter nye ellipseattributter og sender dem til tegningen */
      tegningen.settX(xFelt.getText());
      tegningen.settY(yFelt.getText());
      tegningen.settBredde(breddeFelt.getText());
      tegningen.settHyde(hydeFelt.getText());

      /* Finner fargen til knappen */
      String fargenavn = hendelse.getActionCommand();
      Color farge;
      if (fargenavn.equals("Rd")) farge = Color.red;
      else if (fargenavn.equals("Bl")) farge = Color.blue;
      else farge = Color.cyan;

      /* Setter forgrunnsfargen p tegningen */
      tegningen.setForeground(farge);

      /* Tegner hele vinduet p nytt */
      VinduMedTegning.this.repaint();
    }
  }

  /* Beskriver nordre panel */
  private class BrukerInputPanel extends JPanel {
    public BrukerInputPanel() {
      /* Arg. til GridLayout: Ant. rader og kolonner, horisontal og vertikal avstand */
      setLayout(new GridLayout(4, 2, 5, 5));
      JLabel ledetekst = new JLabel("X-verdi:", JLabel.RIGHT);
      add(ledetekst);
      add(xFelt);

      ledetekst = new JLabel("Y-verdi:", JLabel.RIGHT);
      add(ledetekst);
      add(yFelt);

      ledetekst = new JLabel("Bredde:", JLabel.RIGHT);
      add(ledetekst);
      add(breddeFelt);

      ledetekst = new JLabel("Hyde:", JLabel.RIGHT);
      add(ledetekst);
      add(hydeFelt);
    }
  }

  /* Beskriver sndre panel */
  private class Knappepanel extends JPanel {
    public Knappepanel() {  // Standard layouthndterer FlowLayout
      Knappelytter knappelytteren = new Knappelytter();

      JButton knappRd = new JButton("Rd");
      knappRd.setBackground(Color.red);
      knappRd.addActionListener(knappelytteren);
      add(knappRd);

      JButton knappBl = new JButton("Bl");
      knappBl.setBackground(Color.blue);
      knappBl.addActionListener(knappelytteren);
      add(knappBl);

      JButton knappTurkis = new JButton("Turkis");
      knappTurkis.setBackground(Color.cyan);
      knappTurkis.addActionListener(knappelytteren);
      add(knappTurkis);
    }
  }
}

/*
 * En tegning av en ellipse, der ellipseattributtene kan forandres.
 * Klassen tar inn attributtene som tekster. Dersom en tekst ikke
 * kan omformes til et heltall, settes attributtet lik 0.
 * En liten melding skrives til konsollet.
 */
class Tegning extends JPanel {
  private int x;
  private int y;
  private int bredde;
  private int hyde;
  public Tegning(String startX, String startY, String startBredde, String startHyde) {
    x = omformTekst(startX);
    y = omformTekst(startY);
    bredde = omformTekst(startBredde);
    hyde = omformTekst(startHyde);
  }

  public void settX(String nyX) {
    x = omformTekst(nyX);
  }

  public void settY(String nyY) {
    y = omformTekst(nyY);
  }

  public void settBredde(String nyBredde) {
    bredde = omformTekst(nyBredde);
  }

  public void settHyde(String nyHyde) {
    hyde = omformTekst(nyHyde);
  }

  public void paintComponent(Graphics vindu) {
    vindu.fillOval(x, y, bredde, hyde);
  }

  private int omformTekst(String tekst) {
    try {
      return Integer.parseInt(tekst);
    } catch (NumberFormatException e) {
      System.out.println("Kan ikke omforme " + tekst + " til heltall.");
      return 0;
    }
  }
}

class TestTegneVindu {
  public static void main(String[] args) {
    VinduMedTegning etVindu =
       new VinduMedTegning("Ellipse med brukerbestemt strrelse og plassering");
    etVindu.setVisible(true);
  }
}