/*
 * MedlemImpl.java   E.L. 2002-10-12
 *
 * Filen inneholder klassene MedlemImpl og MedlemsfabrikkImpl.
 *
 * Klassen MedlemImpl er mutabel. Alle data kan endres.
 * Et objekt bestr av et person-objekt og et streng-objekt.
 * Klassen Person er mutabel, her brukes den i parameterlister og
 * som returtype. I og med at overfringen skjer via serialisering, er
 * det imidlertid ingen fare for utilsiktede endringer i dette objektet
 * fra klientens side. Vi forutsetter at tjenerprogrammet heller
 * ikke endrer person-objektet. Vi trenger derfor ikke beskytte objektet
 * ved  lage egne kopier av det (jamfr delkapittel 8.3)
 */

import java.rmi.*;
import java.rmi.server.*;
import mittBibliotek.Person;

class MedlemImpl extends UnicastRemoteObject implements Medlem {
  private Person personen;
  private String adressen;

  public MedlemImpl(Person startPerson, String startAdresse)
                                        throws RemoteException {
    personen = startPerson;
    adressen = startAdresse;
  }

  public synchronized Person finnPerson() throws RemoteException {
    return personen;
  }

  public synchronized void settPerson(Person nyPerson) throws RemoteException {
    personen = nyPerson;
  }

  public synchronized String finnAdresse() throws RemoteException {
    return adressen;
  }

  public synchronized void settAdresse(String nyAdresse) throws RemoteException {
    adressen = nyAdresse;
  }
}

/*
 * Klassen MedlemsfabrikkImpl gjr det mulig for en klient
 *  lage et objekt her p tjenersiden.
 */
class MedlemsfabrikkImpl
      extends UnicastRemoteObject implements Medlemsfabrikk {

  public MedlemsfabrikkImpl() throws RemoteException {
  }

  public Medlem lagMedlem(Person startPerson, String startAdresse)
                                              throws RemoteException {
    return new MedlemImpl(startPerson, startAdresse);
  }
}