/*
 * DatabaseApplikasjon.java  E.L. 2004-09-08
 *
 * Applikasjonen gjr det mulig  vedlikeholde tabellen person.
 * Alle data vises fram, deretter kan brukeren endre, slette eller legge inn nye data.
 * Endringene utfres i databasen, og dersom det gikk bra, i vinduet.
 * Valget "Gjenoppfrisk" henter alle dataene p nytt fra databasen.
 */

import static javax.swing.JOptionPane.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.*;
import mittBibliotek.*;  // Person og PersonDialog

/*
 * Enum-klasse som inneholder de aktuelle knappene.
 */
enum Knapp {
  Slett, Ny, Endre, Gjenoppfrisk;
  private JButton knapp;
  private Knapp() {
    knapp = new JButton(this.name());
  }
  public JButton finnKnapp() {
    return knapp;
  }
}

/*
 * Brukergrensesnittet, dvs. vinduet med liste over navene, og knapper for menyvalg.
 */
class DatabaseGUI extends JFrame {
  private Database databasekontakt;  // settes i klientprogrammet

  private DefaultListModel listeinnhold = new DefaultListModel();
  private JList liste = new JList(listeinnhold);
  private PersonDialog persondialog = new PersonDialog(this);

  public DatabaseGUI(Database startDatabasen) {
    databasekontakt = startDatabasen;
    setTitle("Navneregister");
    setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    addWindowListener(new Vinduslytter());
    add(new OverskriftPanel("Vedlikehold av navneregister"), BorderLayout.NORTH);
    add(new ListePanel(), BorderLayout.CENTER);
    add(new KnappePanel(), BorderLayout.SOUTH);
    fyllListeMedData();  // se nedenfor
    persondialog.setLocation(300, 400);
    setLocation(300, 300);
    pack();
  }

  /* Hjelpemetode: Henter data fra databasen og fyller opp listen p skjermen */
  private void fyllListeMedData() {
    listeinnhold.clear();
    ArrayList<Person> alle = databasekontakt.finnAlle();
    if (alle != null) {
      for (Person enPerson : alle) listeinnhold.addElement(enPerson);
      if (listeinnhold.size() > 0) liste.setSelectedIndex(0); // standard valg
      else { // ingen data, kan ikke trykke p knappene Endre og Slett
        Knapp.Slett.finnKnapp().setEnabled(false);
        Knapp.Endre.finnKnapp().setEnabled(false);
      }
    } else showMessageDialog(DatabaseGUI.this, "Fr ikke hentet data fra databasen.");
  }

  /*
   * N kommer de tre panelene som beskriver brukergrensesnittet.
   */
  private class OverskriftPanel extends JPanel {
    public OverskriftPanel(String overskrift) {
      Font stdSkrift = getFont();
      Font storeBokstaver = new Font(stdSkrift.getName(), stdSkrift.getStyle(), 18);
      JLabel tekst = new JLabel(overskrift);
      tekst.setFont(storeBokstaver);
      add(tekst);
    }
  }

  private class ListePanel extends JPanel {
    public ListePanel() {
      liste.setPreferredSize(
        new Dimension(300, 300)); // FlowLayout tar hensyn til dette
      liste.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
      JScrollPane rullefeltMedListe = new JScrollPane(liste);

      /* Lager tabelloverskrift */
      JViewport jvp = new JViewport(); // se online API-dokumentasjonen
      jvp.setView(new JLabel("Registrerte navn"));
      rullefeltMedListe.setColumnHeader(jvp);

      add(rullefeltMedListe);
    }
  }

  private class KnappePanel extends JPanel {
    public KnappePanel() {
      Knappelytter lytteren = new Knappelytter();
      for (Knapp enKnapp : Knapp.values()) {
        enKnapp.finnKnapp().addActionListener(lytteren);
        add(enKnapp.finnKnapp());
      }
    }
  }

  /*
   * Flgende klasse hndterer alle knappetrykkene:
   */
  private class Knappelytter implements ActionListener {
    public void actionPerformed(ActionEvent hendelse) {
      String kommando = hendelse.getActionCommand();
      Knapp valg = Knapp.valueOf(kommando);

      switch (valg) {
        case Slett:
          Person personen = (Person) liste.getSelectedValue();
          boolean slettet = databasekontakt.slettPerson(personen.finnPersonNr());
          if (slettet) {
            listeinnhold.remove(liste.getSelectedIndex());
            if (listeinnhold.size() == 0) {
              Knapp.Slett.finnKnapp().setEnabled(false);
              Knapp.Endre.finnKnapp().setEnabled(false);
            }
            else liste.setSelectedIndex(0);
          } else {
            showMessageDialog(DatabaseGUI.this, "Fr ikke slettet personen. Kan vre slettet av andre.");
          }
          break;
        case Endre:  // endrer et navn
          personen = (Person) liste.getSelectedValue();
          int indeks = liste.getSelectedIndex();
          if (persondialog.visDialog(personen)) {  // navnet skal endres
            if (databasekontakt.endreNavn(personen)) {
              listeinnhold.set(indeks, personen);  // bytter ut personen
              liste.setSelectedIndex(0);
            } else showMessageDialog(DatabaseGUI.this, "Ingen data endret. Kan vre slettet av andre.");
          }
          break;
        case Ny:  // legger inn en ny person
          personen = new Person(-1,"", "");
          if (persondialog.visDialog(personen)) {
            personen = databasekontakt.registrerNyPerson(personen.finnFornavn(), personen.finnEtternavn());
            showMessageDialog(DatabaseGUI.this, "Personen fr nummer " + personen.finnPersonNr());
            listeinnhold.addElement(personen);
            if (listeinnhold.size() == 1) { // frste personen som er lagt inn
              Knapp.Slett.finnKnapp().setEnabled(true);
              Knapp.Endre.finnKnapp().setEnabled(true);
              liste.setSelectedIndex(0);
            }
          }
          break;
        case Gjenoppfrisk:  // henter "friske" data fra databasen
          fyllListeMedData();
          break;
        default:
          System.out.println("Hit skal ikke programmet komme!");
          break;
      }
    }
  }

  private class Vinduslytter extends WindowAdapter {
    public void windowClosing(WindowEvent hendelse) {
      databasekontakt.kobleNedForbindelse();  // NB!
      System.exit(0);
    }
  }
}

class DatabaseApplikasjon {
  public static void main(String[] args) {
    try {
      String dbDriver = "oracle.jdbc.driver.OracleDriver";
      String dbNavn = "jdbc:oracle:thin:@baron.stud.idb.hist.no:1521:orcl";
      int feilkode = 1;  // returkode fra databasesystemet hvis duplikate primrnkler

      String brukernavn = showInputDialog("Brukernavn: ");
      String passord = showInputDialog("Passord: ");
      Database databasekontakt = new Database(dbDriver, dbNavn, brukernavn, passord, feilkode);

      DatabaseGUI applikasjonen = new DatabaseGUI(databasekontakt);
      applikasjonen.setVisible(true);
    } catch (Exception e) {
      showMessageDialog(null, "Fr ikke koblet opp mot databasen. \nFeilmelding: " + e);
    }
  }
}