/*
 * NedborStatistikk.java   E.L. 2002-07-18
 *
 * Filen inneholder to klasser:
 * Maned: Immutabel klasse som lagrer nedbrdata for en mned. Tilbyr operasjoner 
 * for  analysere disse: finn gjennomsnitt, finn total, finn trre dager, osv.
 * NedborStatistikk: Enkel testklient 
 */

class Maned {
  private String mndNavn;
  private int[] nedbr;

  /*
   * Konstruktren lager en kopi av argumentet med nedbrdata.
   * Klienten kan dermed bruke den samme datastrukturen til  lagre
   * data for forskjellige mneder. Hvert enkelt Maneds-objekt holder
   * dermed rede p sine egne nedbrdata, uavhengig av klientens data.
   */
  public Maned(String startMndNavn, int[] startNedbr) {
    mndNavn = startMndNavn;
    int antDager = startNedbr.length;
    nedbr = new int[antDager];
    for (int i = 0; i < antDager; i++) nedbr[i] = startNedbr[i];
  }

  public String finnMndNavn() {
    return mndNavn;
  }

  public int finnAntDager() {
    return nedbr.length;
  }

  /*
   * Metoden finner nedbren p en gitt dag. Returnerer -1 dersom ugyldig indeks.
   */
  public int finnNedbr(int indeks) {
    if (indeks >= 0 && indeks < nedbr.length) return nedbr[indeks];
    else return -1;  // ugyldig indeks
  }

  /*
   * Metoden finner maksimalnedbren i mneden.
   */
  public int finnMaksimum() {
    int maks = 0;  // Husk at lokale variabler ikke automatisk fr en startverdi.
    if (nedbr.length > 0) {
      maks = nedbr[0];
      for (int i = 1; i < nedbr.length; i++) {
        if (nedbr[i] > maks) maks = nedbr[i];
      }
    }
    return maks;
  }

  /*
   * Metoden finner gjennomsnittlig nedbr i mneden.
   */
  public double finnGjSnitt() {
    int sum = 0;
    for (int i = 0; i < nedbr.length; i++) {
      sum += nedbr[i];
    }
    if (nedbr.length > 0) return (double) sum / (double) nedbr.length;
    else return 0.0;
  }

  /*
   * Metoden finner antall dager uten nedbr.
   */
  public int finnAntTrreDager() {
    int antall = 0;
    for (int i = 0; i < nedbr.length; i++) {
      if (nedbr[i] == 0) antall++;
    }
    return antall;
  }

  /*
   * Metoden finner hvilke dager som har nedbr lik maksimalnedbren for mneden.
   * Returnerer en tabell med indekser til disse dagene.
   */
  public int[] finnDgMaks() {
    int maks = finnMaksimum();

    /* Lager en tabell med plass til alle dagene, det verst tenkelige...*/
    int [] tabell = new int[nedbr.length];
    int antMaks = 0;
    for (int i = 0; i < nedbr.length; i++) {
      if (nedbr[i] == maks) {
        tabell[antMaks] = i;
        antMaks++;
      }
    }
    /* Lager n en tabell med eksakt riktig strrelse og kopierer over. */
    int[] maksDager = new int[antMaks];
    for (int i = 0; i < antMaks; i++) maksDager[i] = tabell[i];
    return maksDager;
  }
}

/*
 * Testprogram av enkleste slag, det vil si at dataene "hardkodes". 
 */
class NedborStatistikk {
  public static void main(String[] args) {
    int[] nedbr = {1, 4, 0, 4, 3};  // liten mned for testforml
    /*int[] nedbr = {};  // har ogs prvd med tabell med lengde 0
     *int[] nedbr = {1};  // og lengde 1 */
    Maned januar = new Maned("Januar", nedbr);
    System.out.println("Statistikk for " + januar.finnMndNavn());
    System.out.println("Maksimum: " + januar.finnMaksimum());
    System.out.println("Gjennomsnitt: " + januar.finnGjSnitt());
    System.out.println("Antall trre dager: " + januar.finnAntTrreDager());

    for (int i = 0; i < januar.finnAntDager(); i++) {
      System.out.println("Nedbr dag nr " + (i + 1) + " er "
        + januar.finnNedbr(i));
    }

    int[] maksDg = januar.finnDgMaks();
    for (int i = 0; i < maksDg.length; i++) {
      System.out.println("Maks. nedbr dag nr: " + (maksDg[i] + 1));
    }
  }
}

/* Kjring av programmet:
Statistikk for Januar
Maksimum: 4
Gjennomsnitt: 2.4
Antall trre dager: 1
Nedbr dag nr 1 er 1
Nedbr dag nr 2 er 4
Nedbr dag nr 3 er 0
Nedbr dag nr 4 er 4
Nedbr dag nr 5 er 3
Maks. nedbr dag nr: 2
Maks. nedbr dag nr: 4
*/