/*
 * SorterOgSoekTabellListerMedComparable.java  E.L. 2006-01-23
 *
 * Filen inneholder to klasser:
 * Flate: Implementerer Comparable, der arealer sammenlignes.
 *        Lager ogs sin egen utgave av equals() som er konsistent med comapreTo()
 * SorterOgSoekTabellListerMedComparable:
 *        Prver metoder fra java.util.Collections og java.util.ArrayList
 */

import java.util.*;

class Flate implements Comparable<Flate> {
  private String navn;  // til identifikasjon
  private double lengde;
  private double bredde;
  private static double toleranse = 0.00001;

  public Flate(String startNavn, double startLengde, double startBredde) {
    navn = startNavn;
    lengde = startLengde;
    bredde = startBredde;
  }

  public String finnNavn() {
    return navn;
  }

  public double finnLengde() {
    return lengde;
  }

  public double finnBredde() {
    return bredde;
  }

  public double finnAreal() {
    return bredde * lengde;
  }

  public double finnOmkrets() {
    return 2 * (lengde + bredde);
  }

  public String toString() {
    java.text.DecimalFormat utskriftFormat = new java.text.DecimalFormat("####0.00");
    return "Flate: " + navn + ", bredde: " + utskriftFormat.format(bredde) +
           " m, lengde: " + utskriftFormat.format(lengde) + " m.";
  }

  /* compareTo() brukes av ske- og sorteringsmetodene i klassene Arrays og Collections */
  public int compareTo(Flate denAndreFlaten) {
    double areal1 = finnAreal();
    double areal2 = denAndreFlaten.finnAreal();
    if (Math.abs(areal2 - areal1) < toleranse) return 0;
    else if (areal1 < areal2) return -1;
    else return 1;
  }

  /* equals() brukes av ske-metodene i klassen ArrayList */
  public boolean equals(Object denAndreFlaten) {
    if (denAndreFlaten instanceof Flate) {
      return (compareTo((Flate) denAndreFlaten) == 0);
    } else return false;
  }
}

class SorterOgSoekTabellListerMedComparable {
  public static void main(String[] args) {

    /* Legger seks flater i en tabell-liste */
    ArrayList<Flate> flater = new ArrayList<Flate>();
    flater.add(new Flate("A", 3, 4));
    flater.add(new Flate("B", 2, 3.5));
    flater.add(new Flate("C", 5, 2));
    flater.add(new Flate("D", 2, 4));
    flater.add(new Flate("E", 4, 3));
    flater.add(new Flate("F", 4, 2.4));

    /* Skriver ut arealene i den opprinnelige rekkeflgen */
    System.out.println("Opprinnelig rekkeflge:");
    for (Flate enFlate : flater) {
      System.out.println(enFlate.finnNavn() + " har areal " + enFlate.finnAreal());
    }

    /*
     * Sker i tabell-listen. Som argument m vi sende inn en referanse til et objekt.
     * For at testen p likhet skal sl ut, m dette objektet ha samme areal som
     * det vi sker etter.
     */
    Flate skeFlate1 = new Flate("sk1", 2, 6); // flate med areal 12 (finnes)
    Flate skeFlate2 = new Flate("sk2", 2, 6.5); // flate med areal 13 (finnes ikke)

    /* Sk i usortert tabell-liste ved hjelp av metoder i klassen ArrayList */
    System.out.println("Resultater sk med indexOf() og lastIndexof(): ");
    int res1 = flater.indexOf(skeFlate1);
    System.out.println("Flate med areal 12 er funnet forfra p indeks " + res1);
    res1 = flater.lastIndexOf(skeFlate1);
    System.out.println("Flate med areal 12 er funnet bakfra p indeks " + res1);
    int res2 = flater.indexOf(skeFlate2);
    System.out.println("Flate med areal 13 finnes ikke, resultat fra sk: " + res2);

    /* Sorterer, og skriver deretter ut i sortert rekkeflge */
    Collections.sort(flater);
    System.out.println("Sortert rekkeflge:");
    for (Flate enFlate : flater) {
      System.out.println(enFlate.finnNavn() + " har areal " + enFlate.finnAreal());
    }

    /* Binrsk, tabell-listen m vre sortert */
    System.out.println("Resultater binrsk: ");
    res1 = Collections.binarySearch(flater, skeFlate1);
    System.out.println("Flate med areal 12 er funnet p indeks " + res1);
    res2 = Collections.binarySearch(flater, skeFlate2); // finnes ikke
    System.out.println("Flate med areal 13 er ikke funnet." +
                       " Kan eventuelt legges inn p indeks " + (-res2 - 1));
  }
}

/* Utskrift:
Opprinnelig rekkeflge:
A har areal 12.0
B har areal 7.0
C har areal 10.0
D har areal 8.0
E har areal 12.0
F har areal 9.6
Resultater sk med indexOf() og lastIndexof():
Flate med areal 12 er funnet forfra p indeks 0
Flate med areal 12 er funnet bakfra p indeks 4
Flate med areal 13 finnes ikke, resultat fra sk: -1
Sortert rekkeflge:
Indeks: 0, navn: B, areal 7.0
Indeks: 1, navn: D, areal 8.0
Indeks: 2, navn: F, areal 9.6
Indeks: 3, navn: C, areal 10.0
Indeks: 4, navn: A, areal 12.0
Indeks: 5, navn: E, areal 12.0
Resultater binrsk:
Flate med areal 12 er funnet p indeks 4
Flate med areal 13 er ikke funnet. Kan eventuelt legges inn p indeks 6
*/
