/*
 * SorterOgSoekTabellerMedComparable.java  E.L. 2006-01-24
 *
 * Filen inneholder to klasser:
 *
 * Flate: Implementerer Comparable, der arealer sammenlignes.
 * Klassen har ogs sin egen utgave av equals() som er konsistent med compareTo()
 *
 * SorterOgSoekTabellerMedComparable: Prver metoder fra java.util.Arrays
 */

import java.util.Arrays;
class Flate implements Comparable<Flate> {
  private String navn;  // til identifikasjon
  private double lengde;
  private double bredde;
  private static double toleranse = 0.00001;

  public Flate(String startNavn, double startLengde, double startBredde) {
    navn = startNavn;
    lengde = startLengde;
    bredde = startBredde;
  }

  public String finnNavn() {
    return navn;
  }

  public double finnLengde() {
    return lengde;
  }

  public double finnBredde() {
    return bredde;
  }

  public double finnAreal() {
    return bredde * lengde;
  }

  public double finnOmkrets() {
    return 2 * (lengde + bredde);
  }

  public String toString() {
    java.text.DecimalFormat utskriftFormat = new java.text.DecimalFormat("####0.00");
    return "Flate: " + navn + ", bredde: " + utskriftFormat.format(bredde) +
           " m, lengde: " + utskriftFormat.format(lengde) + " m.";
  }

  /* compareTo() brukes av ske- og sorteringsmetodene i klassene Arrays og Collections */
  public int compareTo(Flate denAndreFlaten) {
    double areal1 = finnAreal();
    double areal2 = denAndreFlaten.finnAreal();
    if (Math.abs(areal2 - areal1) < toleranse) return 0;
    else if (areal1 < areal2) return -1;
    else return 1;
  }

  /* equals() brukes av ske-metodene i klassen ArrayList */
  public boolean equals(Object denAndreFlaten) {
    if (denAndreFlaten instanceof Flate) {
      return (compareTo((Flate) denAndreFlaten) == 0);
    } else return false;
  }
}

class SorterOgSoekTabellerMedComparable {
  public static void main(String[] args) {

    /* Legger seks flater i en tabell */
    Flate[] flater = {new Flate("A", 3, 4), new Flate("B", 2, 3.5), new Flate("C", 5, 2),
                      new Flate("D", 2, 4), new Flate("E", 4, 3), new Flate("F", 4, 2.4)};

    /* Skriver ut arealene i den opprinnelige rekkeflgen */
    System.out.println("Opprinnelig rekkeflge:");
    for (Flate enFlate : flater) {
      System.out.println(enFlate.finnNavn() + " har arel " + enFlate.finnAreal());
    }

    /* Sorterer, og skriver deretter ut i sortert rekkeflge */
    Arrays.sort(flater);
    System.out.println("Sortert rekkeflge:");
    for (Flate enFlate : flater) {
      System.out.println(enFlate.finnNavn() + " har arel " + enFlate.finnAreal());
    }

    /* I denne sorterte tabellen kan vi ske etter en flate med et bestemt areal */
    Flate skeFlate1 = new Flate("sk", 2, 6); // flate med areal 12
    int res1 = Arrays.binarySearch(flater, skeFlate1); // finnes
    System.out.println("Flate med areal 12 er funnet p indeks " + res1);

    Flate skeFlate2 = new Flate("sk", 2, 6.5); // flate med areal 13
    int res2 = Arrays.binarySearch(flater, skeFlate2); // finnes ikke
    System.out.println("Flate med areal 13 er ikke funnet."
                     + " Kan eventuelt legges inn p indeks " + (-res2 - 1));
  }
}

/* Kjring av programmet:
Opprinnelig rekkeflge:
A har arel 12.0
B har arel 7.0
C har arel 10.0
D har arel 8.0
E har arel 12.0
F har arel 9.6
Sortert rekkeflge:
B har arel 7.0
D har arel 8.0
F har arel 9.6
C har arel 10.0
A har arel 12.0
E har arel 12.0
Flate med areal 12 er funnet p indeks 4
Flate med areal 13 er ikke funnet. Kan eventuelt legges inn p indeks 6
*/
